<?php

define('LDAP_AUTHN_AUTH_MIXED',           0);
define('LDAP_AUTHN_AUTH_EXCLUSIVED',      1);
define('LDAP_AUTHN_CONFLICT_LOG',         0);
define('LDAP_AUTHN_CONFLICT_RESOLVE',     1);
define('LDAP_AUTHN_EMAIL_FIELD_NO',       0);
define('LDAP_AUTHN_EMAIL_FIELD_REMOVE',   1);
define('LDAP_AUTHN_EMAIL_FIELD_DISABLE',  2);
define('LDAP_AUTHN_PROFILE',              'LDAP authentication');
define('LDAP_AUTHN_PROFILE_WEIGHT',       4);

define('LDAP_AUTHN_LOGIN_PROCESS',       variable_get('ldap_authn_login_process', LDAP_AUTHN_AUTH_MIXED));
define('LDAP_AUTHN_LOGIN_CONFLICT',      variable_get('ldap_authn_login_conflict', LDAP_AUTHN_CONFLICT_LOG));
define('LDAP_AUTHN_FORGET_PASSWORDS',    variable_get('ldap_authn_forget_passwords', TRUE));
define('LDAP_AUTHN_SYNC_PASSWORDS',      variable_get('ldap_authn_sync_passwords', FALSE));
define('LDAP_AUTHN_DISABLE_PASS_CHANGE', variable_get('ldap_authn_disable_pass_change', FALSE));
define('LDAP_AUTHN_ALTER_EMAIL_FIELD',   variable_get('ldap_authn_alter_email_field', LDAP_AUTHN_EMAIL_FIELD_NO));
define('LDAP_AUTHN_DEFAULT_USER_ATTR',   variable_get('ldap_authn_default_user_attr', 'uid'));
define('LDAP_AUTHN_DEFAULT_MAIL_ATTR',   variable_get('ldap_authn_default_mail_attr', 'mail'));


/**
 * Main user validation function.
 *
 * If successful, sets the global $user object.
 */
function ldap_authn_login_validate($form, &$form_state) {
  ldap_authn_authenticate($form_state['values']);
}


function _ldap_authn_form_ldap_api_servers_admin_form_alter(&$form, &$form_state) {

  $form['ldap_authn'] = array(
    '#type' => 'fieldset',
    '#title' => t('LDAP Authentication Settings'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );

  $form['ldap_authn']['ldap_authn_status'] = array(
    '#type' => 'checkbox',
    '#title' => t('Use this LDAP server for user authentication?'),
    '#default_value' => 1,
  );
  
  $form['ldap_authn']['ldap_authn_filter_php'] = array(
    '#type' => 'textarea',
    '#title' => t('PHP to filter users based on their LDAP data'),
    '#default_value' => $NULL,
    '#cols' => 25,
    '#rows' => 5,
    '#description' => t('Enter PHP to filter users which are allowed to login based on their LDAP data. Careful, bad PHP code here will break your site. If left empty, no filtering will be done. The code should return TRUE to allow authentication. Following example shows how to disallow users without their homeDirectory set:<br /><code>return isset($ldap[\'homeDirectory\']) && isset($ldap[\'homedirectory\'][0]);</code>'),
  );
  
  $form['#validate'][] = 'ldap_authn_server_data_validate';
  $form['#submit'][] = 'ldap_authn_server_data_submit';
}

function ldap_authn_server_data_validate($form, &$form_state) {
  $values = $form_state['values'];

  if (isset($values['ldap_authn_filter_php']) && !($values['ldap_authn_status'])) {
    drupal_set_message("Code is entered in PHP filter, but server is not selected for use in authentication.
      The code will not be used, until the server is selected for authentication"
      ,'warning');
  }

}

function ldap_authn_server_data_submit($form, &$form_state) {
  // consolidate all ldap_authn module data for this server into one array
  $values = $form_state['values'];
  $ldap_authn_data['status'] = $values['ldap_authn_status'];
  $ldap_authn_data['filter_php'] = $values['ldap_authn_filter_php'];
  $form_state['values']['module_data']['ldap_authn'] = $ldap_authn_data;
  
  //dpm($form_state);
}


function _ldap_authn_user($op, &$edit, &$account, $category = NULL) {

  switch ($op) {
    case 'update':
      if ($category == 'account') {

        // If authentication is being done in "LDAP only" mode, passwords
        // should not be written to the database, or users would be able
        // to log in even after removing their LDAP entry.
        if (isset($account->ldap_authentified) && (LDAP_AUTHN_LOGIN_PROCESS == LDAP_AUTHN_AUTH_EXCLUSIVED || !LDAP_AUTHN_SYNC_PASSWORDS))
          $edit['pass'] = NULL;
      }

      if (LDAP_AUTHN_ALTER_EMAIL_FIELD == LDAP_AUTHN_EMAIL_FIELD_REMOVE)
        unset($edit['mail']);
      break;
    case 'view':
      if (user_access('administer users') && isset($account->ldap_authentified) && $account->ldap_dn) {
        $ldap_server_object = ldap_api_get_server_objects($account->ldap_config);
        $account->content[t(LDAPAUTH_PROFILE)] = array(
         '#type' => 'user_profile_category',
         '#title' => t(LDAP_AUTHN_PROFILE),
         '#attributes' => array('class' => 'ldap-authn-entry'),
         '#weight' => LDAP_AUTHN_PROFILE_WEIGHT,
         'ldap_server' => array('#type' => 'user_profile_item', '#title' => t('LDAP server'), '#value' => l($ldap_server_object->name, 'admin/settings/ldap_servers/edit/'. $ldap_server_object->sid), '#weight' => 0),
         'ldap_dn' => array('#type' => 'user_profile_item', '#title' => t('LDAP dn'), '#value' => $account->ldap_dn, '#weight' => 1),
        );
      }
      break;
  }
}


function _ldap_authn_form_alter(&$form, $form_state, $form_id) {
  global $user;

  // Replace the drupal authenticate function if it's used as validation.
  if (isset($form['#validate']) && is_array($form['#validate']) && ($key = array_search('user_login_authenticate_validate', $form['#validate']))) {
    require_once('ldap_authn.inc');
    $form['#validate'][$key] = 'ldap_authn_login_validate';
  }

  switch ($form_id) {
    case 'user_login_block':
      if (LDAP_AUTHN_DISABLE_PASS_CHANGE)
        unset($form['links']);
      break;
    case 'user_profile_form':
      $account = $form["_account"]["#value"];
      if ($user->uid != 1 && isset($account->ldap_authentified)) {
        if (LDAP_AUTHN_DISABLE_PASS_CHANGE)
        unset($form['account']['pass']);

        switch (LDAP_AUTHN_ALTER_EMAIL_FIELD) {
          case LDAP_AUTHN_EMAIL_FIELD_REMOVE :
            $form['account']['mail']['#type'] = 'hidden';
            $form['account']['mail']['#attributes']['READONLY'] = 'READONLY';
            break;
          case LDAP_AUTHN_EMAIL_FIELD_DISABLE :
            $form['account']['mail']['#attributes']['READONLY'] = 'READONLY';
            break;
        }

        // Remove fieldset if empty.
        if (isset($form['account']) && !isset($form['account']['pass']) && $form['account']['mail']['#type'] == 'hidden' && count(array_filter($form['account'], create_function('$a', 'return is_array($a) ? TRUE : FALSE;'))) == 1) {
          $form['mail'] = $form['account']['mail'];
          unset($form['account']);
        }
      }
      break;
  }
}

/**
 * Main user authentication function.
 *
 * If successful, sets the global $user object.
 */
function ldap_authn_authenticate($form_values = array()) {
  global $user;

  $name = $form_values['name'];
  $pass = trim($form_values['pass']);

  $result = db_query("SELECT name, data, uid FROM {users} WHERE name = '%s' AND uid = '1'", $name);
  $has_users_record = ($user_record = db_fetch_object($result));
  $user_record_data = ($has_users_record) ? unserialize($user_record->data) : NULL;

  // (Design decision) uid=1 (admin user) must always authenticate to local database
  // this user is critical for all drupal admin and upgrade operations so it is best
  // left with drupal's native authentication.

  // CASE 1:  user is user 1 and should bypass 
  if ($user_record->uid == 1) {
    user_authenticate($form_values);
    return;
  }

  // CASE 2: user account name exists in ldap, but account with same name is local user
  if (LDAP_AUTHN_LOGIN_PROCESS == LDAP_AUTHN_AUTH_MIXED) {
    // Authenticate local users first.
    if (!isset($data['ldap_authn_authentified']) || $data['ldap_authn_authentified'] == 0) {
      // A local user with same name exists - authenticate that user.
      if (user_authenticate($form_values)) {
        return;
      }
    }
  }

  // Authenticate LDAP user.
  if (!($result = _ldap_authn_auth($name, $pass)))
    return;
  
  if (!$ldap_user_entry['dn'])
    return;

  $account = user_load(array('name' => $name));
  if (!isset($account->uid)) {

    // Check if the username is allowed.
    if (drupal_is_denied('user', $name)) {
      drupal_set_message(t('The name %name has been denied access.', array('%name' => $name)), 'error');
      return;
    }

  // Register this new user.

    // Generate a random drupal password. LDAP password will be used anyways.
    $pass_new = (LDAP_AUTHN_LOGIN_PROCESS == LDAP_AUTHN_AUTH_EXCLUSIVED || !LDAP_AUTHN_SYNC_PASSWORDS) ? user_password(20) : $pass;

    // If mail attribute is missing, set the name as mail.
    $mail = $ldap_user_entry[$ldap_server_object->mail_attr];


    $userinfo = array('name' => $name, 'pass' => $pass_new, 'mail' => $mail, 'init' => $mail, 'status' => 1, 'authname_ldap_authn' => $name, 'ldap_authentified' => TRUE, 'ldap_dn' => $ldap_user_entry['dn'], 'ldap_config' => $ldap_server_object->sid);
    $user = user_save('', $userinfo);
    watchdog('user', 'New external user %name created from the LDAP server %server.', array('%name' => $name, '%server' => $ldap_server_object->name), WATCHDOG_NOTICE, l(t('edit'), 'user/'. $user->uid .'/edit'));
  
  }
  else if ($account->status == 0) {
    // User is blocked.
    return;
  }
  else {
    // Login existing user.
      
    $data = array(
      'ldap_dn' => $ldap_user_entry['dn'],
      'ldap_config' => $ldap_server_object->sid,
    );

    if (!isset($account->ldap_authentified)) {
      // LDAP and local user conflict.
      if (LDAP_AUTHN_LOGIN_CONFLICT == LDAP_AUTHN_CONFLICT_LOG) {
        watchdog('user', 'LDAP user with DN %dn has a naming conflict with a local drupal user %name', array('%dn' => $ldap_user_entry['dn'], '%name' => $account->name), WATCHDOG_ERROR);
        drupal_set_message(t('Another user already exists in the system with the same login name. You should contact the system administrator in order to solve this conflict.'), 'error');
        return;
      }
      else {
        $data['ldap_authentified'] = TRUE;
        $data['authname_ldap_authn'] = $name;
      }
    }

    // Successfull login.
    // Save the new login data.
    if (LDAP_AUTHN_LOGIN_PROCESS == LDAP_AUTHN_AUTH_MIXED && LDAP_AUTHN_SYNC_PASSWORDS)
      // avoid this: $data['pass'] = $pass;
    $user = user_save($account, $data);
  }

  // Save user's authentication data to the session.
  $_SESSION['ldap_login']['dn'] = $dn;
  // avoid this $_SESSION['ldap_login']['pass'] = $pass;

  user_authenticate_finalize($form_values);
  return $user;
}

/**
 * Authenticate the user against LDAP server.
 *
 * @param $name
 *   A username.
 * @param $pass
 *   A password.
 *
 * @return
 *  on successful logon: array of User's LDAP API ldap object and user's ldap entry
 *  otherwise FALSE
 */
function _ldap_authn_auth($name, $pass) {

  // Don't allow empty passwords because they cause problems on some setups.
  // http://drupal.org/node/87831
  if (empty($pass))
    return FALSE;

  // Cycle through LDAP configurations.  First one to succeed wins.
  $ldap_server_objects = ldap_api_get_server_objects(NULL, 'ldap_authn');
  foreach ($ldap_server_objects as $sid => $ldap_server_object) {
    
    // Look up the user in LDAP.
    $ldap_user_entry = $ldap_server_object->get_ldap_user_entry_from_drupal_username($name);
    if (!($ldap_user_entry) || !isset($ldap_user_entry['dn']))
      continue;

    // Filter users based on their LDAP data.
    if (($code = $ldap_server_object->module_data['ldap_authn']['filter_php']) && !drupal_eval($code))
      continue;

    // Try to authenticate.
    if (!$ldap_server_object->connect($ldap_user_entry['dn'], $pass))
      continue;

    return array($ldap_server_object, $ldap_user_entry);
    
  }

  return FALSE;
}




/**
 * Retrieve the saved ldapgroups saved setting.
 *
 * @param $sid
 *   A server ID or user object.
 * @param $req
 *   An attribute name.
 *
 * @return
 *   The attribute value.
 */
function _ldap_authn_ldap_info($sid, $req) {
  return ldap_api_server_get_attribute($sid, $req);
}